@@ -19,18 +19,27 @@ namespace Botan {
 enum class EC_Point_Format {
    Uncompressed = 0,
    Compressed = 1,
-   Hybrid = 2,
 
    UNCOMPRESSED BOTAN_DEPRECATED("Use EC_Point_Format::Uncompressed") = Uncompressed,
    COMPRESSED BOTAN_DEPRECATED("Use EC_Point_Format::Compressed") = Compressed,
-   HYBRID BOTAN_DEPRECATED("Use EC_Point_Format::Hybrid") = Hybrid,
+
+   Hybrid BOTAN_DEPRECATED("Hybrid point encoding is deprecated") = 2,
+   HYBRID BOTAN_DEPRECATED("Hybrid point encoding is deprecated") = 2
 };
 
 /**
-* This class represents one point on a curve of GF(p)
+* Deprecated elliptic curve type
+*
+* Use EC_AffinePoint in new code
+*
+* This type will be removed/hidden in Botan4
 */
 class BOTAN_PUBLIC_API(2, 0) EC_Point final {
    public:
+      friend class EC_Point_Var_Point_Precompute;
+      friend class EC_Point_Multi_Point_Precompute;
+      friend class EC_Point_Base_Point_Precompute;
+
       typedef EC_Point_Format Compression_Type;
       using enum EC_Point_Format;
 
@@ -72,6 +81,19 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
          return (*this);
       }
 
+      /**
+      * Point multiplication operator
+      *
+      * Simple unblinded Montgomery ladder
+      *
+      * Warning: prefer the functions on EC_Group such as
+      * blinded_var_point_multiply
+      *
+      * @param scalar the scalar value
+      * @return *this multiplied by the scalar value
+      */
+      EC_Point mul(const BigInt& scalar) const;
+
       /**
       * Construct a point from its affine coordinates
       * Prefer EC_Group::point(x,y) for this operation.
@@ -79,7 +101,7 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
       * @param x affine x coordinate
       * @param y affine y coordinate
       */
-      EC_Point(const CurveGFp& curve, const BigInt& x, const BigInt& y);
+      EC_Point(const CurveGFp& curve, BigInt x, BigInt y);
 
       /**
       * EC2OSP - elliptic curve to octet string primitive
@@ -119,54 +141,22 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
          return *this;
       }
 
-      /**
-      * get affine x coordinate
-      * @result affine x coordinate
-      */
-      BigInt get_affine_x() const;
-
-      /**
-      * get affine y coordinate
-      * @result affine y coordinate
-      */
-      BigInt get_affine_y() const;
-
-      /**
-      * Return the internal x coordinate
-      *
-      * Note this may be in Montgomery form
-      */
-      const BigInt& get_x() const { return m_coord_x; }
-
-      /**
-      * Return the internal y coordinate
-      *
-      * Note this may be in Montgomery form
-      */
-      const BigInt& get_y() const { return m_coord_y; }
-
-      /**
-      * Return the internal z coordinate
-      *
-      * Note this may be in Montgomery form
-      */
-      const BigInt& get_z() const { return m_coord_z; }
-
-      void swap_coords(BigInt& new_x, BigInt& new_y, BigInt& new_z) {
-         m_coord_x.swap(new_x);
-         m_coord_y.swap(new_y);
-         m_coord_z.swap(new_z);
-      }
-
       /**
       * Force this point to affine coordinates
+      *
+      * Convert the point to its equivalent affine coordinates. Throws if this
+      * is the point at infinity.
       */
       void force_affine();
 
       /**
       * Force all points on the list to affine coordinates
+      *
+      * Force several points to be affine at once. Uses Montgomery's trick to
+      * reduce number of inversions required, so this is much faster than
+      * calling ``force_affine`` on each point in sequence.
       */
-      static void force_all_affine(std::vector<EC_Point>& points, secure_vector<word>& ws);
+      static void force_all_affine(std::span<EC_Point> points, secure_vector<word>& ws);
 
       bool is_affine() const;
 
@@ -184,12 +174,36 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
       bool on_the_curve() const;
 
       /**
-      * swaps the states of *this and other, does not throw!
-      * @param other the object to swap values with
+      * Return the fixed length big endian encoding of x coordinate
       */
-      void swap(EC_Point& other);
+      secure_vector<uint8_t> x_bytes() const;
 
-      friend void swap(EC_Point& x, EC_Point& y) { x.swap(y); }
+      /**
+      * Return the fixed length big endian encoding of y coordinate
+      */
+      secure_vector<uint8_t> y_bytes() const;
+
+      /**
+      * Return the fixed length concatenation of the x and y coordinates
+      */
+      secure_vector<uint8_t> xy_bytes() const;
+
+      /**
+      * get affine x coordinate
+      * @result affine x coordinate
+      */
+      BigInt get_affine_x() const;
+
+      /**
+      * get affine y coordinate
+      * @result affine y coordinate
+      */
+      BigInt get_affine_y() const;
+
+      /**
+      * Return the zero (aka infinite) point associated with this curve
+      */
+      EC_Point zero() const { return EC_Point(m_curve); }
 
       /**
       * Randomize the point representation
@@ -197,17 +211,62 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
       */
       void randomize_repr(RandomNumberGenerator& rng);
 
+      /**
+      * Equality operator
+      */
+      bool operator==(const EC_Point& other) const;
+
+      bool operator!=(const EC_Point& other) const = default;
+
+      /**
+      * swaps the states of *this and other
+      * @param other the object to swap values with
+      */
+      void swap(EC_Point& other) noexcept;
+
+#if defined(BOTAN_DISABLE_DEPRECATED_FEATURES)
+
+   private:
+      friend class EC_Mul2Table_Data_BN;
+#endif
+
+      /**
+      * Return the internal x coordinate
+      *
+      * Note this may be in Montgomery form
+      */
+      BOTAN_DEPRECATED("Use affine coordinates only") const BigInt& get_x() const { return m_coord_x; }
+
+      /**
+      * Return the internal y coordinate
+      *
+      * Note this may be in Montgomery form
+      */
+      BOTAN_DEPRECATED("Use affine coordinates only") const BigInt& get_y() const { return m_coord_y; }
+
+      /**
+      * Return the internal z coordinate
+      *
+      * Note this may be in Montgomery form
+      */
+      BOTAN_DEPRECATED("Use affine coordinates only") const BigInt& get_z() const { return m_coord_z; }
+
+      BOTAN_DEPRECATED("Deprecated no replacement")
+
+      void swap_coords(BigInt& new_x, BigInt& new_y, BigInt& new_z) {
+         m_coord_x.swap(new_x);
+         m_coord_y.swap(new_y);
+         m_coord_z.swap(new_z);
+      }
+
+      friend void swap(EC_Point& x, EC_Point& y) { x.swap(y); }
+
       /**
       * Randomize the point representation
       * The actual value (get_affine_x, get_affine_y) does not change
       */
       void randomize_repr(RandomNumberGenerator& rng, secure_vector<word>& ws);
 
-      /**
-      * Equality operator
-      */
-      bool operator==(const EC_Point& other) const;
-
       /**
       * Point addition
       * @param other the point to add to *this
@@ -218,11 +277,11 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
 
          const size_t p_words = m_curve.get_p_words();
 
-         add(other.m_coord_x.data(),
+         add(other.m_coord_x._data(),
              std::min(p_words, other.m_coord_x.size()),
-             other.m_coord_y.data(),
+             other.m_coord_y._data(),
              std::min(p_words, other.m_coord_y.size()),
-             other.m_coord_z.data(),
+             other.m_coord_z._data(),
              std::min(p_words, other.m_coord_z.size()),
              workspace);
       }
@@ -248,6 +307,10 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
 
       /**
       * Point addition - mixed J+A
+      *
+      * @warning This function assumes that @p other is affine, if this is not
+      *          correct the result will be invalid.
+      *
       * @param other affine point to add - assumed to be affine!
       * @param workspace temp space, at least WORKSPACE_SIZE elements
       */
@@ -256,9 +319,9 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
          BOTAN_DEBUG_ASSERT(other.is_affine());
 
          const size_t p_words = m_curve.get_p_words();
-         add_affine(other.m_coord_x.data(),
+         add_affine(other.m_coord_x._data(),
                     std::min(p_words, other.m_coord_x.size()),
-                    other.m_coord_y.data(),
+                    other.m_coord_y._data(),
                     std::min(p_words, other.m_coord_y.size()),
                     workspace);
       }
@@ -311,11 +374,6 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
          return x;
       }
 
-      /**
-      * Return the zero (aka infinite) point associated with this curve
-      */
-      EC_Point zero() const { return EC_Point(m_curve); }
-
       /**
       * Return base curve of this point
       * @result the curve over GF(p) of this point
@@ -329,14 +387,6 @@ class BOTAN_PUBLIC_API(2, 0) EC_Point final {
       BigInt m_coord_x, m_coord_y, m_coord_z;
 };
 
-/**
-* Point multiplication operator
-* @param scalar the scalar value
-* @param point the point value
-* @return scalar*point on the curve
-*/
-BOTAN_PUBLIC_API(2, 0) EC_Point operator*(const BigInt& scalar, const EC_Point& point);
-
 /**
 * ECC point multiexponentiation - not constant time!
 * @param p1 a point
@@ -348,11 +398,6 @@ BOTAN_PUBLIC_API(2, 0) EC_Point operator*(const BigInt& scalar, const EC_Point&
 BOTAN_PUBLIC_API(2, 0)
 EC_Point multi_exponentiate(const EC_Point& p1, const BigInt& z1, const EC_Point& p2, const BigInt& z2);
 
-// relational operators
-inline bool operator!=(const EC_Point& lhs, const EC_Point& rhs) {
-   return !(rhs == lhs);
-}
-
 // arithmetic operators
 inline EC_Point operator-(const EC_Point& lhs) {
    return EC_Point(lhs).negate();
@@ -369,13 +414,18 @@ inline EC_Point operator-(const EC_Point& lhs, const EC_Point& rhs) {
 }
 
 inline EC_Point operator*(const EC_Point& point, const BigInt& scalar) {
-   return scalar * point;
+   return point.mul(scalar);
+}
+
+inline EC_Point operator*(const BigInt& scalar, const EC_Point& point) {
+   return point.mul(scalar);
 }
 
 /**
 * Perform point decoding
 * Use EC_Group::OS2ECP instead
 */
+BOTAN_DEPRECATED("Use EC_Group::OS2ECP")
 EC_Point BOTAN_PUBLIC_API(2, 0) OS2ECP(const uint8_t data[], size_t data_len, const CurveGFp& curve);
 
 /**
@@ -388,15 +438,12 @@ EC_Point BOTAN_PUBLIC_API(2, 0) OS2ECP(const uint8_t data[], size_t data_len, co
 * @param curve_a the curve equation a parameter
 * @param curve_b the curve equation b parameter
 */
+BOTAN_DEPRECATED("Use EC_Group::OS2ECP")
 std::pair<BigInt, BigInt> BOTAN_UNSTABLE_API
-OS2ECP(const uint8_t data[], size_t data_len, const BigInt& curve_p, const BigInt& curve_a, const BigInt& curve_b);
+   OS2ECP(const uint8_t data[], size_t data_len, const BigInt& curve_p, const BigInt& curve_a, const BigInt& curve_b);
 
-template <typename Alloc>
-EC_Point OS2ECP(const std::vector<uint8_t, Alloc>& data, const CurveGFp& curve) {
-   return OS2ECP(data.data(), data.size(), curve);
-}
-
-class EC_Point_Var_Point_Precompute;
+BOTAN_DEPRECATED("Use EC_Group::OS2ECP")
+EC_Point BOTAN_UNSTABLE_API OS2ECP(std::span<const uint8_t> data, const CurveGFp& curve);
 
 // The name used for this type in older versions
 typedef EC_Point PointGFp;
